<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:j2="http://jhove2.org/xsd/1.0.0"
	xmlns:containerMD="http://bibnum.bnf.fr/ns/containerMD-v1" 
	xmlns:wrapper="java:org.jhove2.module.display.util.ContainerMDWrapper"
	extension-element-prefixes="wrapper"
	exclude-result-prefixes="j2 wrapper">
	<xsl:output method="xml" version="1.0" omit-xml-declaration="yes" indent="yes"/>

	<xsl:variable name="ArcModuleNode"
		select="/j2:jhove2/j2:feature[@name='FileSource']/j2:features/j2:feature[@name='Modules']/j2:features/
		j2:feature[@name='Module' and @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/ArcModule']/j2:features"/>
	<xsl:variable name="GzipMemberSourceNodes"
		select="/j2:jhove2/j2:feature[@name='FileSource']//j2:feature[@ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/gzip/properties/GzipEntryProperties']"/>
	
	<xsl:variable name="MimeType1"
		select="/j2:jhove2/j2:feature[@name='FileSource']/j2:features/j2:feature[@name='PresumptiveFormats']
		/j2:features/j2:feature[@name='PresumptiveFormat']
		/j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']
		/j2:features/j2:feature[@name='Value']
		/j2:value"/>
	<xsl:variable name="MimeType2"
		select="/j2:jhove2/j2:feature[@name='FileSource']
		/j2:features/j2:feature[@name='ChildSources']
		/j2:features/j2:feature[@name='ChildSource'][1]
		/j2:features/j2:feature[@name='PresumptiveFormats']
		/j2:features/j2:feature[@name='PresumptiveFormat']
		/j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']
		/j2:features/j2:feature[@name='Value']
		/j2:value"/>

	<xsl:variable name="wrapperInstance" select="wrapper:new()"/>
	<xsl:template match="/j2:jhove2/j2:feature[@name='FileSource']/j2:features">
		<!-- CONTAINER MD -->
		<xsl:element name="containerMD:containerMD">
		<!-- For debug purpose  -->
		<!--
			<xsl:element name="containerMD:profile">
				<xsl:value-of select="$MimeType1" />
				<xsl:text>.</xsl:text>
				<xsl:value-of select="$MimeType2" />
				</xsl:element>
		-->
			<xsl:element name="containerMD:container">
				<!-- Attributes -->
				<xsl:attribute name="creationDateTime">
					<xsl:value-of
						select="concat(substring-before(./j2:feature[@name='FileSystemProperties']/j2:features/j2:feature[@name='LastModified']/j2:value,'+'),'Z')"
					/>
				</xsl:attribute>
				<!-- Element fixity -->
				<xsl:if
					test="number(./j2:feature[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/MeasurableSource/Size']/j2:value) &gt; 0">
					<xsl:element name="containerMD:fixity">
						<xsl:attribute name="size">
							<xsl:value-of
								select="./j2:feature[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/MeasurableSource/Size']/j2:value"
							/>
						</xsl:attribute>
					</xsl:element>
				</xsl:if>
				<!-- Element formatDesignation -->
				<xsl:element name="containerMD:formatDesignation">
					<xsl:element name="containerMD:formatName">
						<xsl:choose>
							<xsl:when test="$MimeType2">
								<xsl:value-of select="$MimeType2" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of	select="$MimeType1" />
							</xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<!-- L'élément formatVersion n'est créé que si cette information est disponible -->
					<xsl:if test="$ArcModuleNode/j2:feature[@name='FileVersion']/j2:value">
						<xsl:element name="containerMD:formatVersion">
							<xsl:value-of
								select="substring-before($ArcModuleNode/j2:feature[@name='FileVersion']/j2:value, '.')" />
						</xsl:element>
					</xsl:if>
				</xsl:element>
					
				<!-- Element encoding  -->
				<xsl:if test="$GzipMemberSourceNodes">
					<xsl:element name="containerMD:encoding">
						<!--  Attributes -->
						<xsl:attribute name="type">
							<xsl:text>compression</xsl:text>
						</xsl:attribute>
						<xsl:attribute name="method">
							<xsl:value-of
								select="$GzipMemberSourceNodes[1]/j2:features/j2:feature[@name='CompressionMethod']/j2:features/j2:feature[@name='Description']/j2:value"
							/>
						</xsl:attribute>
						<xsl:attribute name="originalSize">
							<xsl:value-of select="sum($GzipMemberSourceNodes/j2:features/j2:feature[@name='Size']/j2:value)" />
						</xsl:attribute>
					</xsl:element>
				</xsl:if>

				<!-- Element assessmentInformation  -->
				<xsl:element name="containerMD:assessmentInformation">
					<!-- Attributes -->
					<xsl:attribute name="agentName">
						<xsl:text>JHOVE2</xsl:text>
					</xsl:attribute>
					<xsl:attribute name="agentVersion">
						<xsl:value-of
							select="/j2:jhove2/j2:feature[@name='FileSource']/j2:features/j2:feature[@name='Modules']/j2:features/j2:feature[@name='Module' and @ftid='http://jhove2.org/terms/reportable/org/jhove2/core/JHOVE2']//j2:features/j2:feature[@name='Version']/j2:value"
						/>
					</xsl:attribute>
					<!-- Elements -->
					<xsl:element name="containerMD:outcome">
						<xsl:choose>
							<xsl:when
								test="$ArcModuleNode/j2:feature[@name='isValid']/j2:value = 'true'">
								<xsl:text>Valid</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>Not-valid</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<xsl:for-each select="//j2:features/j2:feature[@name='Message']">
						<xsl:element name="containerMD:outcomeDetailNote">
							<xsl:value-of
								select="./j2:value/text()"
							/>
						</xsl:element>
					</xsl:for-each>
				</xsl:element>
				<!-- containerExtension est dans container -->
				<xsl:if test="$MimeType1='application/x-ia-arc' or $MimeType2='application/x-ia-arc' ">
					<xsl:call-template name="containerExtension"/>
				</xsl:if>
			</xsl:element>
			<!-- entries est au même niveau que container -->
			<xsl:if test="$MimeType1='application/x-ia-arc' or $MimeType2='application/x-ia-arc' ">
				<xsl:call-template name="entries"/>
			</xsl:if>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="containerExtension">
		<xsl:element name="containerMD:containerExtension">
			<xsl:element name="containerMD:ARCContainer">
				<xsl:element name="containerMD:fileName">
					<xsl:value-of select="$ArcModuleNode/j2:feature[@name='ArcFileName']/j2:value" />
				</xsl:element>
				<xsl:element name="containerMD:profile">
					<xsl:value-of select="$ArcModuleNode/j2:feature[@name='FileVersion']/j2:value" />
				</xsl:element>
				<xsl:element name="containerMD:originCode">
					<xsl:value-of
						select="//j2:feature[@name='ExtraProperty' and @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcVersionBlockProperties']
							/j2:features/j2:feature[@name='OriginCode']/j2:value"
					/>
				</xsl:element>
				<xsl:element name="containerMD:versionBlockPayload">
					<xsl:value-of
						select="//j2:feature[@name='ExtraProperty' and @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordBaseProperties'][1]
							/j2:features/j2:feature[@name='HasPayload']/j2:value"
					/>
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template name="entries">
		<!-- Collect the data -->
		<!-- only for ArcRecordSource -->
		<xsl:for-each
			select="//j2:feature[@name='ChildSource' and
				./j2:features/j2:feature[@name='ExtraProperties']/j2:features/j2:feature[@name='ExtraProperty' and 
					@ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordBaseProperties'] and not (
				./j2:features/j2:feature[@name='ExtraProperties']/j2:features/j2:feature[@name='ExtraProperty' and 
					@ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcVersionBlockProperties']) ]">

			<!-- Initialize variables -->
            
            <!-- Compression -->
			<xsl:variable name="method"
				select="../../../j2:feature[@name='ExtraProperties']/j2:features
                    /j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/gzip/properties/GzipEntryProperties']/
					j2:features/
                    j2:feature[@name='CompressionMethod']/j2:features/
                    j2:feature[@name='Description']/j2:value"/>

            <!-- Size -->
			<xsl:variable name="size" 
                select="./j2:features/j2:feature[@name='ExtraProperties']/j2:features/
                    j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordBaseProperties']/
                    j2:features/j2:feature[@name='Length']"/>
                
            <!-- DateTime -->
			<xsl:variable name="dateTime" 
                select="./j2:features/j2:feature[@name='ExtraProperties']/j2:features/
                    j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordBaseProperties']/
                    j2:features/j2:feature[@name='RawArchiveDate']"/>
                    
            <!-- Find format -->
			<xsl:variable name="format"
				select="./j2:features/j2:feature[@name='ChildSources']/j2:features/
                    j2:feature[@name='ChildSource'][1]/j2:features/
                    j2:feature[@name='PresumptiveFormats']/j2:features/
                    j2:feature[@name='PresumptiveFormat']/j2:features/
                    j2:feature[@name='NativeIdentifier' and ./j2:features/j2:feature[@name='Namespace']/j2:value='MIME']/
                    j2:features/j2:feature[@name='Value']/j2:value"/>
                
            <!-- Declared mimetype -->
            <xsl:variable name="mimeType"
                select="./j2:features/j2:feature[@name='ExtraProperties']/j2:features/
                    j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordBaseProperties']/
                    j2:features/j2:feature[@name='ContentType']/j2:value"/>
                    
			<xsl:variable name="sourceName" 
                select="./j2:features/j2:feature[@name='ExtraProperties']/j2:features/
                    j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordBaseProperties']/
                    j2:features/j2:feature[@name='Url']/j2:value"/>
                    
			<xsl:variable name="protocolVersion"
				select="./j2:features/j2:feature[@name='ExtraProperties']/j2:features/
                    j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordProperties']/
                    j2:features/j2:feature[@name='ProtocolVersion']/j2:value"/>
                    
			<xsl:variable name="codeResponse"
				select="./j2:features/j2:feature[@name='ExtraProperties']/j2:features/
                    j2:feature[@name='ExtraProperty' and 
                        @ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/properties/ArcRecordProperties']/
                    j2:features/j2:feature[@name='ProtocolResultCode']/j2:value"/>
					
			<!-- JAVA -->
			<xsl:value-of
				select="wrapper:addEntry($wrapperInstance,$sourceName,$size,$dateTime,$format,$mimeType,$protocolVersion,$codeResponse)"/>

                <xsl:if test="$method != ''">
				<xsl:value-of
					select="wrapper:handleEncoding($wrapperInstance,'compression',$method)"/>
			</xsl:if>
		</xsl:for-each>

		<!-- Print the statistics -->
		<xsl:if
			test="number($ArcModuleNode/j2:feature[@name='ArcRecordNumber']/j2:value) != 0">
			<xsl:element name="containerMD:entries">
				<xsl:element name="containerMD:entriesInformation">
					<xsl:attribute name="number">
						<xsl:value-of
							select="$ArcModuleNode/j2:feature[@name='ArcRecordNumber']/j2:value"
						/>
					</xsl:attribute>
					<xsl:attribute name="minimumSize">
						<xsl:value-of select="wrapper:getMinimumSize($wrapperInstance)"/>
					</xsl:attribute>
					<xsl:attribute name="maximumSize">
						<xsl:value-of select="wrapper:getMaximumSize($wrapperInstance)"/>
					</xsl:attribute>
                    <xsl:if test="wrapper:getFirstDateTime($wrapperInstance) != ''">
						<xsl:attribute name="firstDateTime">
							<xsl:value-of select="wrapper:getFirstDateTime($wrapperInstance)"/>
						</xsl:attribute>
                    </xsl:if>
                    <xsl:if test="wrapper:getLastDateTime($wrapperInstance) != ''">
						<xsl:attribute name="lastDateTime">
							<xsl:value-of select="wrapper:getLastDateTime($wrapperInstance)"/>
						</xsl:attribute>
                    </xsl:if>
					<xsl:attribute name="globalSize">
						<xsl:value-of select="wrapper:getGlobalSize($wrapperInstance)"/>
					</xsl:attribute>
					<xsl:if test="wrapper:hasFormats($wrapperInstance) = 'true'">
						<xsl:element name="containerMD:formats">
							<xsl:value-of select="wrapper:getFormats($wrapperInstance)"
								disable-output-escaping="yes"/>
						</xsl:element>
					</xsl:if>
					<xsl:if test="wrapper:hasEncodings($wrapperInstance) = 'true'">
						<xsl:element name="containerMD:encodings">
							<xsl:value-of select="wrapper:getEncodings($wrapperInstance)"
								disable-output-escaping="yes"/>
						</xsl:element>
					</xsl:if>
					<xsl:element name="containerMD:entriesExtension">
						<xsl:element name="containerMD:ARCEntries">
							<xsl:if test="wrapper:hasDeclaredMimeTypes($wrapperInstance) = 'true'">
								<xsl:element name="containerMD:declaredMimeTypes">
									<xsl:value-of
										select="wrapper:getDeclaredMimeTypes($wrapperInstance)"
										disable-output-escaping="yes"/>
								</xsl:element>
							</xsl:if>
							<xsl:if test="wrapper:hasHosts($wrapperInstance) = 'true'">
								<xsl:element name="containerMD:hosts">
									<xsl:value-of select="wrapper:getHosts($wrapperInstance)"
										disable-output-escaping="yes"/>
								</xsl:element>
							</xsl:if>
							<xsl:if test="wrapper:hasResponses($wrapperInstance) = 'true'">
								<xsl:element name="containerMD:responses">
									<xsl:value-of select="wrapper:getResponses($wrapperInstance)"
										disable-output-escaping="yes"/>
								</xsl:element>
							</xsl:if>
						</xsl:element>
					</xsl:element>
				</xsl:element>
			</xsl:element>
		</xsl:if>
			
	</xsl:template>		
</xsl:stylesheet>
